#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <Servo.h>

// Defines Trig and Echo pins of the Ultrasonic Sensor
const int trigPin = 10;
const int echoPin = 11;
// Variables for the duration and the distance
long duration;
int distance;
Servo myServo; // Creates a servo object for controlling the servo motor
Servo extraServo; // Creates a servo object for controlling the extra servo motor

// LED and Buzzer pin constants
const int BUZZER_PIN = 8;
const int LED_RED_PIN = 2;
const int LED_YELLOW_PIN = 3;
const int LED_GREEN_PIN = 4;
const int LED_BLUE_PIN = 5;

// LCD Display constants
const int I2C_ADDRESS = 0x27;
const int LCD_COLUMNS = 16;
const int LCD_ROWS = 2;

LiquidCrystal_I2C lcd(I2C_ADDRESS, LCD_COLUMNS, LCD_ROWS);

bool isRedLedOn = false; // Track if the red LED is on or off

void setup() {
  pinMode(trigPin, OUTPUT); // Sets the trigPin as an Output
  pinMode(echoPin, INPUT); // Sets the echoPin as an Input

  pinMode(LED_RED_PIN, OUTPUT);
  pinMode(LED_YELLOW_PIN, OUTPUT);
  pinMode(LED_GREEN_PIN, OUTPUT);
  pinMode(LED_BLUE_PIN, OUTPUT);
  pinMode(BUZZER_PIN, OUTPUT);

  Serial.begin(9600);

  myServo.attach(12); // Defines on which pin the servo motor is attached
  extraServo.attach(13); // Defines on which pin the extra servo motor is attached

  lcd.begin(LCD_COLUMNS, LCD_ROWS); // Initializes the LCD display
  lcd.backlight(); // Turns on the backlight of the LCD

  lcd.setCursor(0, 0);
  lcd.print("Distance: ");
  lcd.setCursor(0, 1);
  lcd.print("Angle: ");
  lcd.setCursor(0, 2);
  lcd.print("Status: ");
}

void loop() {
  for (int i = 15; i <= 165; i++) {
    myServo.write(i);
    
    // Check if the red LED is on
    if (isRedLedOn) {
      extraServo.write(i / 2); // Control the extra servo movement with faster rotation speed
    } else {
      extraServo.write(0); // Stop the extra servo when the red LED is off
    }

    delay(30);
    distance = calculateDistance();

    updateDisplay(distance, i);

    // Check the distance and activate LEDs and buzzer accordingly
    if (distance >= 1 && distance <= 10) {
      digitalWrite(LED_RED_PIN, HIGH);
      digitalWrite(LED_YELLOW_PIN, LOW);
      digitalWrite(LED_GREEN_PIN, LOW);
      digitalWrite(LED_BLUE_PIN, LOW);
      if (!digitalRead(BUZZER_PIN)) {
        tone(BUZZER_PIN, 1000);
      }
      lcd.setCursor(8, 2);
      lcd.print("Danger     ");
      isRedLedOn = true; // Set the red LED status to ON
    } else if (distance >= 11 && distance <= 19) {
      digitalWrite(LED_RED_PIN, LOW);
      digitalWrite(LED_YELLOW_PIN, HIGH);
      digitalWrite(LED_GREEN_PIN, LOW);
      digitalWrite(LED_BLUE_PIN, LOW);
      if (!digitalRead(BUZZER_PIN)) {
        tone(BUZZER_PIN, 500);
      }
      lcd.setCursor(8, 2);
      lcd.print("Caution    ");
      isRedLedOn = false; // Set the red LED status to OFF
    } else if (distance >= 20 && distance <= 35) {
      digitalWrite(LED_RED_PIN, LOW);
      digitalWrite(LED_YELLOW_PIN, LOW);
      digitalWrite(LED_GREEN_PIN, LOW);
      digitalWrite(LED_BLUE_PIN, HIGH);
      if (!digitalRead(BUZZER_PIN)) {
        tone(BUZZER_PIN, 250);
      }
      lcd.setCursor(8, 2);
      lcd.print("Warning    ");
      isRedLedOn = false; // Set the red LED status to OFF
    } else {
      digitalWrite(LED_RED_PIN, LOW);
      digitalWrite(LED_YELLOW_PIN, LOW);
      digitalWrite(LED_GREEN_PIN, HIGH);
      digitalWrite(LED_BLUE_PIN, LOW);
      if (digitalRead(BUZZER_PIN)) {
        noTone(BUZZER_PIN);
      }
      lcd.setCursor(8, 2);
      lcd.print("Safe       ");
      isRedLedOn = false; // Set the red LED status to OFF
    }
  }
}

int calculateDistance() {
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);

  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  duration = pulseIn(echoPin, HIGH);

  // Calculate the distance (in cm)
  int distance = duration * 0.034 / 2;

  return distance;
}

void updateDisplay(int distance, int angle) {
  lcd.setCursor(10, 0);
  lcd.print("     "); // Clear the previous distance value
  lcd.setCursor(10, 0);
  lcd.print(distance);

  lcd.setCursor(7, 1);
  lcd.print("   "); // Clear the previous angle value
  lcd.setCursor(7, 1);
  lcd.print(angle);

  delay(100);
}
